#include "dport_mm.h"

//	平台相关头文件
#include "sys.h"
#include "delay.h"
#include "uart_adapt.h"


#define		DEV_NUM		1

#define		BUF_SIZE	1024

#define		DP_DEBUG(fmt, args...)	//aw_kprintf("[DP]:"fmt"\r\n", ##args);

/**
 * 当前平台的read接口没有帧超时的话DELAY_TIME_MS就得加一点延时再去读数据
 * 如：#define		DELAY_TIME_MS			20
 */
#define		DELAY_TIME_MS			50
#define		DELAY_MS(x)				delay_ms(x);

#define		AT_ENTER	"++++++AT\r\n"
#define		AT_EXIT		"------AT\r\n"
#define		AT_OK		"AT+OK\r\n"


ring_t *ringList[DEV_NUM];
static int dev_cnt = 0;

/**
 * 设备创建
 * @param name	名称
 * @return	成功返回句柄，失败返回负数
 */
int dev_creat(char *name)
{
	ringList[dev_cnt++] = ring_buffer_init(BUF_SIZE, 1);
	return dev_cnt-1;
}

/**
 * 设备读数据
 * @param fd	句柄
 * @param buf	缓存
 * @param len	长度
 * @return	成功返回读到的长度，失败返回负数
 */
int dev_read(int fd, uint8_t *buf, uint32_t len)
{
	int ret;
	ret = uart_read(buf, len);
	if(ret < 0 ){
		DP_DEBUG("%s read error (%d).", __FUNCTION__, ret);
	}
	return ret;
}

/**
 * 设备写数据
 * @param fd	句柄
 * @param buf	缓存
 * @param len	长度
 * @return 成功返回写入的长度，失败返回负数
 */
int dev_write(int fd, uint8_t *buf, uint32_t len)
{
	int ret;
	ret = uart_write(buf, len);
	if(ret < 0 ){
		DP_DEBUG("%s send error (%d).", __FUNCTION__, ret);
	}
	return ret;
}

/**
 * 设备帧结构读数据
 * @param fd	句柄
 * @param port 源端口
 * @param buf	缓存
 * @param len	长度
 * @return	成功返回读到的长度，失败返回负数
 */
int dev_frame_read(int fd, transparency_port_t *port, uint8_t *buf, uint32_t len)
{
	int ret;
	uint8_t val;
	uint32_t src = 0;
	uint32_t data_len = 0;
	uint32_t pos = 0;
	
	uint32_t start = 0;
	uint32_t now = 0;
	uint32_t cnt = 0;
	
 	int flag = 0;
	uint8_t recv_buf[BUF_SIZE];
	
	memset(recv_buf, 0, sizeof(recv_buf));
	ret = dev_read(fd, recv_buf, sizeof(recv_buf));
	if(ret < 0 ){
		DP_DEBUG("%s read error (%d).", __FUNCTION__, ret);
	}else if(ret == 0)return 0;
		
	
	ring_buffer_write(ringList[fd], recv_buf, ret);
	while(ring_buffer_available(ringList[fd]))
	{
		ring_buffer_read(ringList[fd], &val, 1);
		recv_buf[now++] = val;
		
		if(val == 'A' && flag == 0){
			start = now - 1;
			cnt = 0;
			flag++;
		}else if(val == 'T' && flag == 1)flag++;
		else if(val == '+' && flag == 2)flag++;
		else if(val == 'R' && flag == 3)flag++;
		else if(val == 'E' && flag == 4)flag++;
		else if(val == 'A' && flag == 5)flag++;
		else if(val == 'D' && flag == 6)flag++;
		else if(val == '=' && flag == 7)flag++;
		else if(flag == 8){
			if(val > '0' && val < '9'){
				if(src)src *= 10;
				src += (val - '0');
			}else if(val == ',')flag++;
			
		}else if(flag == 9){
			if(val > '0' && val < '9'){
				if(data_len)data_len *= 10;
				data_len += (val - '0');
			}else if(val == ',')flag++;
			
		}else if(flag == 10){
			if(pos != data_len)buf[pos++] = val;
			if(pos == data_len){
				flag++;
			}
		}else if(val == '\r' && flag == 11){
			flag++;
		}else if(val == '\n' && flag == 12)break;
		else flag = 0;
		
		if(flag)cnt++;
	}
	
	if(flag == 12){
		*port = (transparency_port_t)(1 << (src - 1));
		return data_len;
	}else if(flag > 0){
		ring_buffer_write(ringList[fd], &recv_buf[start], cnt);
		
	}
	return 0;
}

char* tar_to_str(int tar, char *str);
int check_ok(uint8_t *str);
/**
 * 设备帧结构写数据
 * @param fd	句柄
 * @param tar	目标端口
 * @param buf	缓存
 * @param len	长度
 * @return 成功返回写入的长度，失败返回负数
 */
int dev_frame_write(int fd, transparency_tar_t tar, uint8_t *buf, uint32_t len)
{
	int ret;
	uint32_t str_len;
	char *temp;
	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];
	
	memset(send_buf, 0, sizeof(send_buf));
	str_len = sprintf((char*)send_buf, "AT+SEND=");

	temp = tar_to_str(tar, (char*)(send_buf + str_len));
	
	sprintf((char*)temp, ",%d,",
						len);
	
	str_len = strlen((char*)send_buf);
	
	ret = dev_write(fd, send_buf, str_len);
	if(ret < 0 ){
		DP_DEBUG("%s send error (%d).", __FUNCTION__, ret);
	}
	
	ret = dev_write(fd, buf, len);
	if(ret < 0 ){
		DP_DEBUG("%s send error (%d).", __FUNCTION__, ret);
	}
	
	ret = dev_write(fd, (uint8_t*)"\r\n", 2);
	if(ret < 0 ){
		DP_DEBUG("%s send error (%d).", __FUNCTION__, ret);
	}
	
	ret = dev_read(fd, recv_buf, BUF_SIZE);
	if(ret <= 0){
		DP_DEBUG("%s read error (%d).", __FUNCTION__, ret);

	}
	
	if(!check_ok(recv_buf))return -1;
	
	return ret;
}


/**
 * 发送然后读取数据
 * @param fd	句柄
 * @param sbuf	发送缓存
 * @param slen	发送长度
 * @param rbuf	接收缓存
 * @param rlen	接收长度
 * @return
 */
int dp_send_then_read(int fd, uint8_t *sbuf, uint32_t slen, uint8_t *rbuf, uint32_t rlen)
{
	int ret = -1;
	uint8_t recv_buf[BUF_SIZE];
	DP_DEBUG("write result: %s",sbuf);
	
	DELAY_MS(DELAY_TIME_MS);
	ret = dev_write(fd, sbuf, slen);
	if(ret < 0 || (uint32_t)ret != slen){
		DP_DEBUG("%s send error (%d).", __FUNCTION__, ret);
		return -1;
	}

	//	若读接口无帧超时则延时一下
	DELAY_MS(DELAY_TIME_MS);

	//	读取设备回复
	ret = dev_read(fd, recv_buf, BUF_SIZE);
	if(ret <= 0){
		DP_DEBUG("%s read error (%d).", __FUNCTION__, ret);

	}else{
		recv_buf[ret] = 0;
		DP_DEBUG("read result: %s",recv_buf);
		memcpy(rbuf, recv_buf, ret);
		return ret;
	}

	return ret;

}

int check_ok(uint8_t *str)
{
	if(strstr((char*)str, "AT+OK\r\n"))return 1;
	else return 0;
}

/**
 * 检测ip是否正确
 * @param str	字符串
 * @param buf	ip缓存
 * @return
 */
int check_ip(uint8_t* val, uint8_t buf[4])
{
	uint8_t ip[4];
	char *str = (char*)val;
	char *temp = strstr(str, "AT+GET=");
	if(!temp){
		DP_DEBUG("%s fail.", __FUNCTION__);
		return 0;
	}

	str += strlen("AT+GET=");


	{
	    int count = 0; // 记录已经遇到的数字段数
	    int value = 0; // 记录当前段的数值
	    uint8_t hasDot = 0; // 记录是否已经有了一个点号

	    if (str == NULL) {
	        return 0;
	    }

	    for (int i = 0; str[i] != '\0'; ++i) {
	        if (str[i] >= '0' && str[i] <= '9') { // 如果字符是数字
	            if (value > 255 / 10) { // 如果之前的数字加上这个新数字可能超过255，则不合法
	                return 0;
	            }
	            value = value * 10 + (str[i] - '0'); // 更新数值
	            hasDot = 1; // 重置hasDot，因为我们正在读取一个数字
	        } else if (str[i] == '.') { // 如果字符是点号
	            if (count >= 3 || !hasDot) { // 如果已经有三个点号，或者前一个字符不是数字，则不合法
	                return 0;
	            }
	            if (value > 255) { // 如果当前段的数值大于255，则不合法
	                return 0;
	            }
	            ip[count] = value;
	            value = 0; // 重置value，准备读取下一段数字
	            hasDot = 0; // 设置hasDot为真，表示我们遇到了一个点号
	            ++count; // 增加数字段计数
	        } else { // 如果字符既不是数字也不是点号
	            break; // 不合法
	        }
	    }

	    ip[count] = value;
	    // 检查最后一个段是否有效且字符串以数字结束
	    if (value > 255 || !hasDot) {
	        return 0;
	    }

	    // 如果成功遍历完字符串且符合规则，则返回1
	    if(count == 3){
	    	memcpy(buf, ip, 4);
	    	return 1;
	    }else{
	    	return 0;
	    }
	}


}


int check_mode(uint8_t* val, net_mode_t *mode)
{
	char *str = (char*)val;
	char *temp = strstr(str, "AT+GET=");
	if(!temp){
		DP_DEBUG("%s fail.", __FUNCTION__);
		return 0;
	}

	str += strlen("AT+GET=");
	if(!strncmp(str, "common", strlen("common")))*mode = MODE_COMMON;
	else if(!strncmp(str, "bonding", strlen("bonding")))*mode = MODE_BONDING;
	else if(!strncmp(str, "bridgeif", strlen("bridgeif")))*mode = MODE_BRIDGEIF;
	else return 0;

	return 1;
}

int check_number(uint8_t* val, int *num)
{
	char *str = (char*)val;
	char *temp = strstr(str, "AT+GET=");
	if(!temp){
		DP_DEBUG("%s fail.", __FUNCTION__);
		return 0;
	}

	str += strlen("AT+GET=");
	*num = atoi(str);
	return 1;
}

int check_enable(uint8_t* val, enable_t *enable)
{
	char *str = (char*)val;
	char *temp = strstr(str, "AT+GET=");
	if(!temp){
		DP_DEBUG("%s fail.", __FUNCTION__);
		return 0;
	}

	str += strlen("AT+GET=");
	if(!strncmp(str, "true", strlen("true")))*enable = ENABLE_TRUE;
	else if(!strncmp(str, "false", strlen("false")))*enable = ENABLE_FALSE;
	else return 0;

	return 1;
}

int check_atcmd(uint8_t* val, transparency_atcmd_t *atcmd)
{
	char *str = (char*)val;
	char *temp = strstr(str, "AT+GET=");
	if(!temp){
		DP_DEBUG("%s fail.", __FUNCTION__);
		return 0;
	}

	str += strlen("AT+GET=");
	if(!strncmp(str, "Enable", strlen("Enable")))*atcmd = ATCMD_ENABLE;
	else if(!strncmp(str, "Disable", strlen("Disable")))*atcmd = ATCMD_DISABLE;
	else return 0;

	return 1;
}

int check_transparency_mode(uint8_t* val, transparency_mode_t *mode)
{
	char *str = (char*)val;
	char *temp = strstr(str, "AT+GET=");
	if(!temp){
		DP_DEBUG("%s fail.", __FUNCTION__);
		return 0;
	}

	str += strlen("AT+GET=");
	if(!strncmp(str, "transparency", strlen("transparency")))*mode = MODE_TRANSPARENCY;
	else if(!strncmp(str, "frame", strlen("frame")))*mode = MODE_FRAME;
	else return 0;

	return 1;
}

uint32_t read_tar_bit(char *str){
	char *p;
	uint32_t tar_bit = 0;
	char temp[BUF_SIZE];

	memset(temp, 0, BUF_SIZE);
	strcpy(temp, str);
	p = strtok(temp, "|");
	while(p){
		int num = atoi(p);
		if(num>0)tar_bit |= 1<<(num-1);
		p = strtok(NULL, "|");
	}

	return tar_bit;
}

int check_tar(uint8_t* val, int *tar)
{
	char *str = (char*)val;
	char *temp = strstr(str, "AT+GET=");
	if(!temp){
		DP_DEBUG("%s fail.", __FUNCTION__);
		return 0;
	}

	str += strlen("AT+GET=");
	*tar = read_tar_bit(str);

	return 1;
}

/**
 * 进入协议模式
 * @param fd
 * @return	成功返回0,失败返回负数
 */
int dp_enter(int fd)
{
	int ret = -1;
	uint8_t recv_buf[BUF_SIZE];

	ret = dp_send_then_read(fd, (uint8_t*)AT_ENTER, strlen(AT_ENTER), recv_buf, BUF_SIZE);

	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;

}

/**
 * 退出协议模式
 * @param fd
 * @return 成功返回0,失败返回负数
 */
int dp_exit(int fd)
{
	int ret = -1;
	uint8_t recv_buf[BUF_SIZE];

	ret = dp_send_then_read(fd, (uint8_t*)AT_EXIT, strlen(AT_EXIT), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}

/**
 * 设置模式
 * @param fd	句柄
 * @param mode	模式
 * @return 成功返回0，失败返回负数
 */
int dp_set_mode(int fd, net_mode_t mode)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=net,mode,%s\r\n",
				mode_str[mode]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}

/**
 * 获取模式
 * @param fd	句柄
 * @param mode	模式
 * @return	成功返回0，失败返回负数
 */
int dp_get_mode(int fd, net_mode_t *mode)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=net,mode\r\n");

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_mode(recv_buf, mode)){
		DP_DEBUG("%s error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}


/**
 *	设置网卡信息
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_set_net(int fd, int index, net_cfg_t *cfg)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=eth%d,ip,%d.%d.%d.%d\r\n",
						index,
						cfg->ip[0],
						cfg->ip[1],
						cfg->ip[2],
						cfg->ip[3]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set ip error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=eth%d,mask,%d.%d.%d.%d\r\n",
						index,
						cfg->mask[0],
						cfg->mask[1],
						cfg->mask[2],
						cfg->mask[3]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set mask error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=eth%d,gateway,%d.%d.%d.%d\r\n",
						index,
						cfg->gateway[0],
						cfg->gateway[1],
						cfg->gateway[2],
						cfg->gateway[3]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set gateway error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}


/**
 * 获取网卡信息
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_get_net(int fd, int index, net_cfg_t *cfg)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=eth%d,ip\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ip(recv_buf, cfg->ip)){
		DP_DEBUG("%s get ip error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=eth%d,mask\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ip(recv_buf, cfg->mask)){
		DP_DEBUG("%s get mask error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=eth%d,gateway\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ip(recv_buf, cfg->gateway)){
		DP_DEBUG("%s get gateway error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}

/**
 * 设置串口
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_set_uart(int fd, int index, uart_cfg_t *cfg)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=uart%d,baudrate,%d\r\n",
						index,
						cfg->baudrate);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set baudrate error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=uart%d,bytesize,%d\r\n",
						index,
						uart_bytesize_map[cfg->bytesize]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set bytesize error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=uart%d,parity,%d\r\n",
						index,
						uart_parity_map[cfg->parity]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set parity error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=uart%d,stopbits,%d\r\n",
						index,
						uart_stopbits_map[cfg->stopbits]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set stopbits error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}

/**
 * 获取串口配置
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_get_uart(int fd, int index, uart_cfg_t *cfg)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=uart%d,baudrate\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->baudrate)){
		DP_DEBUG("%s get baudrate error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=uart%d,bytesize\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, (int*)&cfg->bytesize)){
		DP_DEBUG("%s get bytesize error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=uart%d,parity\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, (int*)&cfg->parity)){
		DP_DEBUG("%s get parity error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=uart%d,stopbits\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, (int*)&cfg->stopbits)){
		DP_DEBUG("%s get stopbits error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}

/**
 * 设置socket
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_set_socket(int fd, int index, socket_cfg_t *cfg)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,mode,%d\r\n",
						index,
						cfg->mode);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set mode error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,host,%d.%d.%d.%d\r\n",
						index,
						cfg->host[0],
						cfg->host[1],
						cfg->host[2],
						cfg->host[3]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set host error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,multicast,%d.%d.%d.%d\r\n",
						index,
						cfg->multicast[0],
						cfg->multicast[1],
						cfg->multicast[2],
						cfg->multicast[3]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set multicast error (%d).", __FUNCTION__, ret);
		return -1;
	}


	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,enable,%s\r\n",
						index,
						enable_str_map[cfg->enable]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set enable error (%d).", __FUNCTION__, ret);
		return -1;
	}
//////////////////////////////////////////////
	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,port,%d\r\n",
						index,
						cfg->port);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set port error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,max_client_nr,%d\r\n",
						index,
						cfg->max_client_nr);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set max_client_nr error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,keepalive_time,%d\r\n",
						index,
						cfg->keepalive_time);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set keepalive_time error (%d).", __FUNCTION__, ret);
		return -1;
	}
//////////////////
	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,idle_down_time,%d\r\n",
						index,
						cfg->idle_down_time);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set idle_down_time error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=socket%d,retry_interval,%d\r\n",
						index,
						cfg->retry_interval);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set retry_interval error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;

}

/**
 * 获取socket配置
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_get_socket(int fd, int index, socket_cfg_t *cfg)
{
	int ret = -1;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,mode\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, (int*)&cfg->mode)){
		DP_DEBUG("%s get mode error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,host\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ip(recv_buf, cfg->host)){
		DP_DEBUG("%s get host error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,multicast\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ip(recv_buf, cfg->multicast)){
		DP_DEBUG("%s get multicast error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,enable\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_enable(recv_buf, &cfg->enable)){
		DP_DEBUG("%s get enable error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,port\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->port)){
		DP_DEBUG("%s get port error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,max_client_nr\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->max_client_nr)){
		DP_DEBUG("%s get max_client_nr error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,keepalive_time\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->keepalive_time)){
		DP_DEBUG("%s get keepalive_time error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,idle_down_time\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->idle_down_time)){
		DP_DEBUG("%s get idle_down_time error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=socket%d,retry_interval\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->retry_interval)){
		DP_DEBUG("%s get retry_interval error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}

char* tar_to_str(int tar, char *str)
{
	int len;

	for(int i = 0; i< 15; i++){
		if(tar & (1<<i)){
			len = sprintf(str, "%d|", i+1);
			str += len;
		}
	}
	str -= 1;
	*str = 0 ;
	return str;
}

/**
 * 设置传输
 * @param fd	句柄
 * @param port	端口
 * @param cfg	配置
 * @return 成功返回0,失败返回负数
 */
int dp_set_transparency(int fd, transparency_port_t port, transparency_cfg_t *cfg)
{
	int ret = -1;
	int len;
	char *temp;
	int index;

	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	for(int i= 0; i < 16 ;i++){
		if(port & (1<<i)){
			index = i + 1;
			break;
		}
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=transparency%d,mode,%s\r\n",
						index,
						transparency_str_map[cfg->mode]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set mode error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=transparency%d,at_cmd,%s\r\n",
						index,
						atcmd_str_map[cfg->at_cmd]);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set at_cmd error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	len = sprintf((char*)send_buf, "AT+SET=transparency%d,tar_number,",
						index);

	temp = tar_to_str(cfg->tar_number, (char*)(send_buf + len));

	sprintf(temp, "\r\n");

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set tar_number error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=transparency%d,pack_size,%d\r\n",
						index,
						cfg->pack_size);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set pack_size error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+SET=transparency%d,time_out,%d\r\n",
						port,
						cfg->time_out);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_ok(recv_buf)){
		DP_DEBUG("%s set time_out error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);

	return 0;
}

/**
 * 获取传输端口配置
 * @param fd	句柄
 * @param port	端口
 * @param cfg	配置
 * @return	成功返回0,失败返回负数
 */
int dp_get_transparency(int fd, transparency_port_t port, transparency_cfg_t *cfg)
{
	int ret = -1;
	int index;
	uint8_t send_buf[BUF_SIZE];
	uint8_t recv_buf[BUF_SIZE];

	for(int i= 0; i < 16 ;i++){
		if(port & (1<<i)){
			index = i + 1;
			break;
		}
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=transparency%d,mode\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_transparency_mode(recv_buf, &cfg->mode)){
		DP_DEBUG("%s get mode error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=transparency%d,at_cmd\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_atcmd(recv_buf, &cfg->at_cmd)){
		DP_DEBUG("%s get at_cmd error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=transparency%d,tar_number\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_tar(recv_buf, &cfg->tar_number)){
		DP_DEBUG("%s get tar_number error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=transparency%d,pack_size\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->pack_size)){
		DP_DEBUG("%s get pack_size error (%d).", __FUNCTION__, ret);
		return -1;
	}

	memset(send_buf, 0 , BUF_SIZE);
	memset(recv_buf, 0 , BUF_SIZE);
	sprintf((char*)send_buf, "AT+GET=transparency%d,time_out\r\n",
						index);

	ret = dp_send_then_read(fd, send_buf, strlen((char*)send_buf), recv_buf, BUF_SIZE);
	if(ret < 0 || !check_number(recv_buf, &cfg->time_out)){
		DP_DEBUG("%s get time_out error (%d).", __FUNCTION__, ret);
		return -1;
	}

	DP_DEBUG("%s success.", __FUNCTION__);
	return 0;

}

