#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "sys.h"
#include "usart.h"
#include "timer.h"

#include "ring.h"


#define	MEM_ALLOC	malloc
#define	MEM_FREE	free



int ring_buffer_available(ring_t *ring) {
    if (ring == NULL) {
        return -1; 
    }
    return (ring->tailPos - ring->headPos + ring->size) % ring->size;
}



ring_t* ring_buffer_init(uint32_t size, uint8_t overWriteFlag) {
	
	ring_t	*ring;

	if(!size)return NULL;

	ring = MEM_ALLOC(sizeof(ring_t) + size);
	if(!ring)return NULL;

	ring->buffer = (uint8_t*)(ring + 1);
	ring->size = size;
	ring->headPos = 0;
	ring->tailPos = 0;
	ring->overWriteFlag = overWriteFlag;

	return ring;
	
}

int ring_buffer_clear(ring_t *ring) {
	if (!ring)return -1;

	MEM_FREE(ring);
	
	return 0;
}

int ring_buffer_write(ring_t *ring, uint8_t *data, uint32_t length) 
{
		uint32_t freeSpace;
		uint32_t firstPartSize;
    if (ring == NULL || data == NULL || length == 0 || ring->size == 0) {  
        return -1; 
    }  
  
    
    if (length > ring->size) {  
       
        if (!ring->overWriteFlag) {  
            return -1; 
        }  
      
        length = ring->size;  
    }  
  
    
    freeSpace = (ring->headPos >= ring->tailPos) ?  
        (ring->size - (ring->headPos - ring->tailPos)) :  
        (ring->tailPos - ring->headPos);  
  
    
    if (length > freeSpace) {  

        if (!ring->overWriteFlag) {  
            return -1; 
        }  
      
        length = freeSpace;  
    }  
  

    firstPartSize = (ring->tailPos + length <= ring->size) ?  
        length : (ring->size - ring->tailPos);  
  

    memcpy(ring->buffer + ring->tailPos, data, firstPartSize);  
  

    if (length > firstPartSize) {  
        memcpy(ring->buffer, (uint8_t *)data + firstPartSize, length - firstPartSize);  
    }  
  

    ring->tailPos = (ring->tailPos + length) % ring->size;  
		ring->count += length;
		if(ring->count > ring->size)ring->count = ring->size;
  
 
    if (ring->overWriteFlag && ring->count == ring->size && ring->tailPos > ring->headPos) {  
        ring->headPos = ring->tailPos;
    }  
    return 0;  
}

int ring_buffer_read(ring_t *ring, uint8_t *data, uint32_t size) 
{
		uint8_t *readPtr;
		uint32_t read = 0; 
	
    if (!ring || !data || size == 0) {  
        return -1; 
    }  
		
    if(!ring->count)return 0;
		
    if (size > ring->count) {  
        size = ring->count;
    }  
      
    readPtr = (uint8_t*)data;  
    read = 0;
      
    while (size > 0) {  
        uint32_t end = (ring->headPos + size <= ring->size) ?   
                        (ring->headPos + size) :   
                        ring->size;
        uint32_t chunk = (end > ring->headPos) ?   
                         (end - ring->headPos) :   
                         (ring->size - ring->headPos);  
          
          
        memcpy(readPtr + read, ring->buffer + ring->headPos, chunk);  
          
        
        ring->headPos = (ring->headPos + chunk) % ring->size;  
          
        size -= chunk;  
        read += chunk;  
    }  
    ring->count -= read;
    return read; 
}

